@@ -1,23 +1,23 @@
-import socket
-import json
-import mimetypes
 from .socketutils import ServerSocket
-##from .extensions import Protocol_HTTP
 
 class Hook:
     def __init__(self,name,controller=None):
         self.name=name
         self.controller=controller or self._call
         self.functions=[]
+        self.default=None
         self.topfunctions=[] ## Top functions override all the others. These are sorted by priority, and must return "True" or "False" (determining whether or not to continue)
     def _call(self,*args,**kwargs):
-        continu=True
-        for x in self.topfunctions:
+        if len(self.functions)+len(self.topfunctions)>0: ## Allow for a "default function" which will only run if nothing else is available. So far, no one has used new controller functions!
+            continu=True
+            for x in self.topfunctions:
+                if continu:
+                    continu=x(*args,**kwargs)
             if continu:
-                continu=x(*args,**kwargs)
-        if continu:
-            for x in self.functions:
-                x(*args,**kwargs)
+                for x in self.functions:
+                    x(*args,**kwargs)
+        elif self.default:
+            self.default(*args,**kwargs)
     def call(self,*args,**kwargs):
         self.controller(*args,**kwargs)
     def addFunction(self,function):
@@ -32,41 +32,46 @@ class Hook:
         self.topfunctions.remove(function)
     def delFunction(self,function):
         self.functions.remove(function)
+    def setDefaultFunction(self,function):
+        self.default=function
+    def doesAnything(self):
+        if len(self.topfunctions)+len(self.functions)>0:
+            return True
+        return False
 
 
 class TCPServer:
-    def __init__(self,host,port,blocking=True):
-        self.server=ServerSocket(host,port)
+    def __init__(self,host,port,blocking=True,*args,**kwargs):
+        self.server=ServerSocket(host,port,blocking=blocking)
         self.blocking=blocking
         self.host=host
         self.port=port
-        self.extensions=[]
-        self.protocols=[]
+        self.extensions={}
+        self.functable={}
+        self.protocols={}
         self.hooks={}
         init=self.addHook('init')
         init.addFunction(self.listen)
         main=self.addHook("mainloop")
         main.addFunction(self.run)
+        main.addFunction(self.tasks)
         handle=self.addHook("handle")
         handle.addFunction(self.handle)
-    def inittasks(self):
+        self.inittasks(*args,**kwargs)
+    def inittasks(self,*args,**kwargs):
         pass
     def listen(self,lst=5):
         self.server.listen(lst)
+    def tasks(self):
+        pass
     def addExtension(self,extensionobject):
-        self.extensions.append(extensionobject)
-        extensionobject.extend(self)
+        self.protocols[extensionobject.addToServer(self)]=extensionobject
     def addProtocol(self,protocolObject):
-        self.protocols.append(protocolObject)
-        protocolObject.addToServer(self)
+        self.protocols[protocolObject.addToServer(self)]=protocolObject
     def run(self):
-        if self.blocking:
-            connection=self.server.wait_until_connection()
-            self.getHook("handle").call(connection,connection.recieveall())
-        else:
-            connection=self.server.get_connection_nonblocking()
-            if connection:
-                self.getHook("handle").call(connection,connection.recieveall())
+        connection=self.server.get_connection()
+        data=connection.recvall()
+        if connection: self.getHook("handle").call(connection,data)
     def getHook(self,hook):
         return self.hooks[hook]
     def addHook(self,hook):
@@ -76,8 +81,14 @@ class TCPServer:
     def delHook(self,hook):
         del self.hooks[hook]
     def handle(self,connection,data):
-        print(data)
+        pass
     def start(self,*args,**kwargs):
         self.getHook("init").call(*args,**kwargs)
         while 1:
             self.getHook("mainloop").call() ## Mainloop functions must not have args
+    def addFuncToTable(self,name,function):
+        self.functable[name]=function
+    def callFuncFromTable(self,name,*args,**kwargs):
+        self.functable[name](*args,**kwargs)
+    def delFuncFromTable(self,name):
+        del self.functable[name]
